require "vpsb/version"
require 'executable'
require 'awesome_print'

module Vpsb
  module Commands
    class Base
      include Executable

      def go(*params)
        setup(*params)
        build(*params)
        cook(*params)
        deploy(*params)
      end

      def init(*params)
        Init.new.execute(*params)
      end

      def setup(*params)
        Setup.new(core).execute(*params)
      end

      def build(*params)
        Build.new(core).execute(*params)
      end

      def cook(*params)
        Cook.new(core).execute(*params)
      end

      def deploy(*params)
        Deploy.new(core).execute(*params)
      end

      def help!
        puts Vpsb::Commands::HelpMessages.show
        exit
      end
      alias :h! :help!

      def core
        @core ||= (
          c = Core.new
          Vpsb::Resources.constants.each do |k|
            klass = Vpsb::Resources.const_get(k)
            c.register_resource klass.new if klass.is_a?(Class)
          end
          c
        )
      end

      def call(name=nil, *params)
        puts Vpsb::Commands::HelpMessages.intro
        case name.to_s.strip
          when 'go',     'g' then go(params)
          when 'init',   'i' then init(params)
          when 'deploy', 'd' then deploy(params)
          else puts Vpsb::Commands::HelpMessages.unknown_command
        end
      end
    end
  end
end