module Vpsb
  class Core
    class MissingResourceError < StandardError; end
    def get(name, opts={})
      name = name.to_sym
      return data[name] = resources[name].call(self) if opts[:force]
      data[name] ||= get_resource(name, opts).call(self)
    end

    def present?(name)
      !blank?(name)
    end

    def blank?(name)
      data[name].nil?
    end

    def resources
      @resources ||= {}
    end

    def data
      @data ||= {}
    end

    def register_resources(*resource_list)
      resource_list.each{ |r| register_resource(r) }
    end

    def register_resource(resource)
      resources[resource.name] = resource
    end

    private

    def get_resource(name, opts)
      raise MissingResourceError.new("Resource '#{name}', not found") unless resources[name]
      resources[name]
    end
  end
end